@@ -32,6 +32,8 @@ import six
 
 from octavia.api.handlers import abstract_handler
 from octavia.common import constants
+from octavia.common import rpc
+
 
 cfg.CONF.import_group('oslo_messaging', 'octavia.common.config')
 
@@ -46,11 +48,10 @@ class BaseProducer(abstract_handler.BaseObjectHandler):
 
     def __init__(self):
         topic = cfg.CONF.oslo_messaging.topic
-        self.transport = messaging.get_rpc_transport(cfg.CONF)
         self.target = messaging.Target(
             namespace=constants.RPC_NAMESPACE_CONTROLLER_AGENT,
             topic=topic, version="1.0", fanout=False)
-        self.client = messaging.RPCClient(self.transport, target=self.target)
+        self.client = rpc.get_client(self.target)
 
     def create(self, model):
         """Sends a create message to the controller via oslo.messaging
@@ -108,6 +109,39 @@ class LoadBalancerProducer(BaseProducer):
         method_name = "delete_{0}".format(self.payload_class)
         self.client.cast({}, method_name, **kw)
 
+    def failover(self, data_model):
+        """sends a failover message to the controller via oslo.messaging
+
+        :param data_model:
+        """
+        model_id = getattr(data_model, 'id', None)
+        p_class = self.payload_class
+        kw = {"{0}_id".format(p_class): model_id}
+        method_name = "failover_{0}".format(self.payload_class)
+        self.client.cast({}, method_name, **kw)
+
+
+class AmphoraProducer(BaseProducer):
+    """Sends failover messages to the RPC end of the queue consumer
+
+    """
+    PAYLOAD_CLASS = "amphora"
+
+    @property
+    def payload_class(self):
+        return self.PAYLOAD_CLASS
+
+    def failover(self, data_model):
+        """sends a failover message to the controller via oslo.messaging
+
+        :param data_model:
+        """
+        model_id = getattr(data_model, 'id', None)
+        p_class = self.payload_class
+        kw = {"{0}_id".format(p_class): model_id}
+        method_name = "failover_{0}".format(self.payload_class)
+        self.client.cast({}, method_name, **kw)
+
 
 class ListenerProducer(BaseProducer):
     """Sends updates,deletes and creates to the RPC end of the queue consumer
@@ -152,6 +186,21 @@ class MemberProducer(BaseProducer):
     def payload_class(self):
         return self.PAYLOAD_CLASS
 
+    def batch_update(self, old_ids, new_ids, updated_models):
+        """sends an update message to the controller via oslo.messaging
+
+        :param old_ids: list of member ids that are being deleted
+        :param new_ids: list of member ids that are being created
+        :param updated_models: list of member model objects to update
+        """
+        updated_dicts = [m.to_dict(render_unsets=False)
+                         for m in updated_models]
+        kw = {"old_{0}_ids".format(self.payload_class): old_ids,
+              "new_{0}_ids".format(self.payload_class): new_ids,
+              "updated_{0}s".format(self.payload_class): updated_dicts}
+        method_name = "batch_update_{0}s".format(self.payload_class)
+        self.client.cast({}, method_name, **kw)
+
 
 class L7PolicyProducer(BaseProducer):
     """Sends updates,deletes and creates to the RPC end of the queue consumer
@@ -181,11 +230,12 @@ class ProducerHandler(abstract_handler.BaseHandler):
     used to send messages via the Class variables load_balancer, listener,
     health_monitor, member, l7policy and l7rule.
     """
-
-    load_balancer = LoadBalancerProducer()
-    listener = ListenerProducer()
-    pool = PoolProducer()
-    health_monitor = HealthMonitorProducer()
-    member = MemberProducer()
-    l7policy = L7PolicyProducer()
-    l7rule = L7RuleProducer()
+    def __init__(self):
+        self.load_balancer = LoadBalancerProducer()
+        self.listener = ListenerProducer()
+        self.pool = PoolProducer()
+        self.health_monitor = HealthMonitorProducer()
+        self.member = MemberProducer()
+        self.l7policy = L7PolicyProducer()
+        self.l7rule = L7RuleProducer()
+        self.amphora = AmphoraProducer()
