@@ -68,7 +68,7 @@ class L7PolicyController(base.BaseController):
 
         query_filter = self._auth_get_all(context, project_id)
 
-        db_l7policies, links = self.repositories.l7policy.get_all(
+        db_l7policies, links = self.repositories.l7policy.get_all_API_list(
             context.session, show_deleted=False,
             pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
             **query_filter)
