@@ -70,7 +70,7 @@ class PoolsController(base.BaseController):
 
         query_filter = self._auth_get_all(context, project_id)
 
-        db_pools, links = self.repositories.pool.get_all(
+        db_pools, links = self.repositories.pool.get_all_API_list(
             context.session, show_deleted=False,
             pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
             **query_filter)
