@@ -16,25 +16,51 @@
 
 import re
 
+import six
 from sqlalchemy.orm import collections
 
 from octavia.common import constants
 
 
 class BaseDataModel(object):
-
-    # NOTE(brandon-logan) This does not discover dicts for relationship
-    # attributes.
-    def to_dict(self):
+    def to_dict(self, calling_classes=None, recurse=False, **kwargs):
         """Converts a data model to a dictionary."""
+        calling_classes = calling_classes or []
         ret = {}
         for attr in self.__dict__:
-            if attr.startswith('_'):
+            if attr.startswith('_') or not kwargs.get(attr, True):
                 continue
-            if isinstance(getattr(self, attr), (BaseDataModel, list)):
-                ret[attr] = None
+            value = self.__dict__[attr]
+
+            if recurse:
+                if isinstance(getattr(self, attr), list):
+                    ret[attr] = []
+                    for item in value:
+                        if isinstance(item, BaseDataModel):
+                            if type(self) not in calling_classes:
+                                ret[attr].append(
+                                    item.to_dict(calling_classes=(
+                                        calling_classes + [type(self)])))
+                            else:
+                                ret[attr] = None
+                        else:
+                            ret[attr] = item
+                elif isinstance(getattr(self, attr), BaseDataModel):
+                    if type(self) not in calling_classes:
+                        ret[attr] = value.to_dict(
+                            calling_classes=calling_classes + [type(self)])
+                    else:
+                        ret[attr] = None
+                elif six.PY2 and isinstance(value, six.text_type):
+                    ret[attr.encode('utf8')] = value.encode('utf8')
+                else:
+                    ret[attr] = value
             else:
-                ret[attr] = self.__dict__[attr]
+                if isinstance(getattr(self, attr), (BaseDataModel, list)):
+                    ret[attr] = None
+                else:
+                    ret[attr] = value
+
         return ret
 
     def __eq__(self, other):
@@ -194,8 +220,9 @@ class HealthMonitor(BaseDataModel):
     def __init__(self, id=None, project_id=None, pool_id=None, type=None,
                  delay=None, timeout=None, fall_threshold=None,
                  rise_threshold=None, http_method=None, url_path=None,
-                 expected_codes=None, enabled=None, pool=None,
-                 provisioning_status=None, name=None):
+                 expected_codes=None, enabled=None, pool=None, name=None,
+                 provisioning_status=None, operating_status=None,
+                 created_at=None, updated_at=None):
         self.id = id
         self.project_id = project_id
         self.pool_id = pool_id
@@ -210,7 +237,10 @@ class HealthMonitor(BaseDataModel):
         self.enabled = enabled
         self.pool = pool
         self.provisioning_status = provisioning_status
+        self.operating_status = operating_status
         self.name = name
+        self.created_at = created_at
+        self.updated_at = updated_at
 
     def delete(self):
         self.pool.health_monitor = None
@@ -286,7 +316,7 @@ class Member(BaseDataModel):
                  protocol_port=None, weight=None, enabled=None,
                  subnet_id=None, operating_status=None, pool=None,
                  created_at=None, updated_at=None, provisioning_status=None,
-                 name=None):
+                 name=None, monitor_address=None, monitor_port=None):
         self.id = id
         self.project_id = project_id
         self.pool_id = pool_id
@@ -301,6 +331,8 @@ class Member(BaseDataModel):
         self.updated_at = updated_at
         self.provisioning_status = provisioning_status
         self.name = name
+        self.monitor_address = monitor_address
+        self.monitor_port = monitor_port
 
     def delete(self):
         for mem in self.pool.members:
@@ -399,6 +431,17 @@ class LoadBalancer(BaseDataModel):
         self.created_at = created_at
         self.updated_at = updated_at
 
+    def update(self, update_dict):
+        for key, value in update_dict.items():
+            if key == 'vip':
+                if self.vip is not None:
+                    self.vip.update(value)
+                else:
+                    value.update({'load_balancer_id': self.id})
+                    self.vip = Vip(**value)
+            else:
+                setattr(self, key, value)
+
 
 class VRRPGroup(BaseDataModel):
 
@@ -417,12 +460,15 @@ class VRRPGroup(BaseDataModel):
 class Vip(BaseDataModel):
 
     def __init__(self, load_balancer_id=None, ip_address=None,
-                 subnet_id=None, port_id=None, load_balancer=None):
+                 subnet_id=None, network_id=None, port_id=None,
+                 load_balancer=None, qos_policy_id=None):
         self.load_balancer_id = load_balancer_id
         self.ip_address = ip_address
         self.subnet_id = subnet_id
+        self.network_id = network_id
         self.port_id = port_id
         self.load_balancer = load_balancer
+        self.qos_policy_id = qos_policy_id
 
 
 class SNI(BaseDataModel):
@@ -454,7 +500,7 @@ class Amphora(BaseDataModel):
                  ha_ip=None, vrrp_port_id=None, ha_port_id=None,
                  load_balancer=None, role=None, cert_expiration=None,
                  cert_busy=False, vrrp_interface=None, vrrp_id=None,
-                 vrrp_priority=None):
+                 vrrp_priority=None, cached_zone=None):
         self.id = id
         self.load_balancer_id = load_balancer_id
         self.compute_id = compute_id
@@ -471,6 +517,7 @@ class Amphora(BaseDataModel):
         self.load_balancer = load_balancer
         self.cert_expiration = cert_expiration
         self.cert_busy = cert_busy
+        self.cached_zone = cached_zone
 
     def delete(self):
         for amphora in self.load_balancer.amphorae:
@@ -489,9 +536,10 @@ class AmphoraHealth(BaseDataModel):
 
 class L7Rule(BaseDataModel):
 
-    def __init__(self, id=None, l7policy_id=None, type=None,
+    def __init__(self, id=None, l7policy_id=None, type=None, enabled=None,
                  compare_type=None, key=None, value=None, l7policy=None,
-                 invert=False, provisioning_status=None):
+                 invert=False, provisioning_status=None, operating_status=None,
+                 project_id=None, created_at=None, updated_at=None):
         self.id = id
         self.l7policy_id = l7policy_id
         self.type = type
@@ -501,6 +549,11 @@ class L7Rule(BaseDataModel):
         self.l7policy = l7policy
         self.invert = invert
         self.provisioning_status = provisioning_status
+        self.operating_status = operating_status
+        self.project_id = project_id
+        self.created_at = created_at
+        self.updated_at = updated_at
+        self.enabled = enabled
 
     def delete(self):
         if len(self.l7policy.l7rules) == 1:
@@ -519,7 +572,9 @@ class L7Policy(BaseDataModel):
     def __init__(self, id=None, name=None, description=None, listener_id=None,
                  action=None, redirect_pool_id=None, redirect_url=None,
                  position=None, listener=None, redirect_pool=None,
-                 enabled=None, l7rules=None, provisioning_status=None):
+                 enabled=None, l7rules=None, provisioning_status=None,
+                 operating_status=None, project_id=None, created_at=None,
+                 updated_at=None):
         self.id = id
         self.name = name
         self.description = description
@@ -533,6 +588,10 @@ class L7Policy(BaseDataModel):
         self.enabled = enabled
         self.l7rules = l7rules or []
         self.provisioning_status = provisioning_status
+        self.operating_status = operating_status
+        self.project_id = project_id
+        self.created_at = created_at
+        self.updated_at = updated_at
 
     def _conditionally_remove_pool_links(self, pool):
         """Removes links to the given pool from parent objects.
@@ -561,10 +620,9 @@ class L7Policy(BaseDataModel):
                 self.redirect_url = None
                 pool = self._find_in_graph('Pool' + value)
                 self.redirect_pool = pool
-                if len(self.l7rules) > 0 and (self.enabled is True or
-                                              ('enabled' in update_dict.keys()
-                                               and update_dict['enabled']
-                                               is True)):
+                if len(self.l7rules) > 0 and (self.enabled is True or (
+                        'enabled' in update_dict.keys() and
+                        update_dict['enabled'] is True)):
                     if pool not in self.listener.pools:
                         self.listener.pools.append(pool)
                     if self.listener not in pool.listeners:
@@ -584,15 +642,15 @@ class L7Policy(BaseDataModel):
                 self.listener.l7policies.insert(value - 1, self)
             elif key == 'enabled':
                 if (value is True and self.action ==
-                        constants.L7POLICY_ACTION_REDIRECT_TO_POOL
-                        and self.redirect_pool is not None
-                        and len(self.l7rules) > 0
-                        and self.redirect_pool not in self.listener.pools):
+                        constants.L7POLICY_ACTION_REDIRECT_TO_POOL and
+                        self.redirect_pool is not None and
+                        len(self.l7rules) > 0 and
+                        self.redirect_pool not in self.listener.pools):
                     self.listener.pools.append(self.redirect_pool)
                     self.redirect_pool.listeners.append(self.listener)
                 elif (value is False and self.action ==
-                        constants.L7POLICY_ACTION_REDIRECT_TO_POOL
-                        and self.redirect_pool is not None):
+                        constants.L7POLICY_ACTION_REDIRECT_TO_POOL and
+                        self.redirect_pool is not None):
                     self._conditionally_remove_pool_links(
                         self.redirect_pool)
             setattr(self, key, value)
