@@ -258,6 +258,7 @@ def subnet_exists(subnet_id):
 
 def qos_policy_exists(qos_policy_id):
     network_driver = utils.get_network_driver()
+    qos_extension_enabled(network_driver)
     try:
         qos_policy = network_driver.get_qos_policy(qos_policy_id)
     except Exception:
@@ -266,6 +267,12 @@ def qos_policy_exists(qos_policy_id):
     return qos_policy
 
 
+def qos_extension_enabled(network_driver):
+    if not network_driver.qos_enabled():
+        raise exceptions.ValidationException(detail=_(
+            "VIP QoS policy is not allowed in this deployment."))
+
+
 def network_exists_optionally_contains_subnet(network_id, subnet_id=None):
     """Raises an exception when a network does not exist.
 
